#include "CppOsTest.h"

using namespace std;

namespace testing
{
	bool failed;
	std::vector<Test *> AllTests;
	int passCount;
	std::vector<Test *> Fails;
	EOut ErrorOutput;
	TEOut ThrowErrorOutput;
	DOut DummyOutput;
	
	Test::Test()
	{
	}
	
	Test::~Test()
	{
		if (FullName)
			delete[] FullName;
	}

	void Test::SetUp()
	{
	}
	
	void Test::TearDown()
	{
	}

	const char *Test::GetFullName()
	{
		return FullName;
	}

	void Test::SetName()
	{
		if (!CaseName || !TestName)
			return;

		FullName = new char[strlen(CaseName) + 1 + strlen(TestName) + 1];
		strcpy(FullName, CaseName);
		strcat(FullName, ".");
		strcat(FullName, TestName);
	}
	
	void AddTest(Test *tc)
	{
		AllTests.push_back(tc);
	}

	static void RunTest(Test &t)
	{
		cout << "[ RUN      ] " << t.GetFullName() << endl;

		try
		{
			t.SetUp();
			t.RunTest();
		}
		catch (...)
		{
		}
		t.TearDown();

		if (failed)
		{
			cout << "[  FAILED  ] " << t.GetFullName() << endl;
			Fails.push_back(&t);
		}
		else
		{
			cout << "[       OK ] " << t.GetFullName() << endl;
			passCount++;
		}
	}
	
	static void PrintResults()
	{
		cout << "[  PASSED  ] " << ::testing::passCount << " tests." << endl;
		if (::testing::Fails.size() != 0)
		{
			cout << "[  FAILED  ] " << ::testing::Fails.size() << " tests, listed below:" << endl;
			for (int i = 0; i < ::testing::Fails.size(); i++)
			{
				cout << "[  FAILED  ] " << ::testing::Fails[0]->GetFullName() << endl;
			}
		}	
	}
	
	int doRun()
	{
		for (int i = 0; i < AllTests.size(); i++)
		{
			failed = false;
			Test &t = *AllTests[i];
			
			RunTest(t);
		}
		
		PrintResults();
		
		return Fails.size() != 0;
	}
	
	void hasFailed()
	{
		failed = true;
	}

	Out *eqFail(const char *expected, const char *actual, int line, char *file, bool fail)
	{
		hasFailed();

		cout << file << ":" << line << ": Failure" << endl;
		cout << "Value of: " << actual << endl;
		cout << "Expected: " << expected << endl;

		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}

	Out *genericFail(const char *x, const char *y, const char *symbol, int line, char *file, bool fail)
	{
		hasFailed();

		cout << file << ":" << line << ": Failure" << endl;
		cout << "Expected: " << "(" << x << ") " << symbol << " (" << y << "), actual: " << x << " vs " << y << endl;

		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *booleanFail(bool actual, bool expected, const char *condition, int line, char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		cout << "Value of: " << condition << endl;
		cout << "  Actual: " << (actual ? "true" : "false") << endl;
		cout << "Expected: " << (expected ? "true" : "false") << endl;
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	char *strtolower(const char *str1)
	{
		int len = strlen(str1);
		char *str2 = new char[len];
		for (int i = 0; i < len; i++)
		{
			str2[i] = tolower(str1[i]);
		}
		str2[len] = 0;
		
		return str2;
	}
	
	bool doStringTest(const char *str1, const char *str2, bool equality, bool ignoreCase)
	{
		if (!str1)
		{
			if (!str2)
			{
				if (equality)
					return true;
				return false;
			}
			
			if (equality)
				return false;
			return true;
		}
		
		if (ignoreCase)
		{
			str1 = strtolower(str1);
			str2 = strtolower(str2);
		}
		
		int result = strcmp(str1, str2);
		
		if (ignoreCase)
		{
			delete[] str1;
			delete[] str2;
		}
		
		if (equality)
			return result == 0;
		return result != 0;
	}
	
	Out *stringFail(const char *expected, const char *actual, bool equality, bool ignoreCase, int line, char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		if (equality)
		{
			cout << "Value of: " << actual << endl;
			cout << "Expected: " << expected << (ignoreCase ? " (ignoring case)" : "") << endl;
		}
		else
		{
			cout << "Expected: " << "(" << expected << ") !=  (" << actual << "), actual: " << expected << " vs " << actual << (ignoreCase ? " (ignoring case)" : "") << endl;
		}
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	
	}
	
	Out *excFail(const char *expr, const char *exc, int line, char *file, bool fail)
	{
		hasFailed();
		cout << file << ":" << line << ": Failure" << endl;
		cout << "Expected: " << expr << " throws an exception of type " << exc << "." << endl;
		cout << "  Actual: it throws nothing." << endl;
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}

	Out *excFail2(const char *expr, int line, char *file, bool fail)
	{
		hasFailed();
		cout << file << ":" << line << ": Failure" << endl;
		cout << "Expected: " << expr << " throws an exception." << endl;
		cout << "  Actual: it doesn't." << endl;
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *excFail3(const char *expr, int line, char *file, bool fail)
	{
		hasFailed();
		cout << file << ":" << line << ": Failure" << endl;
		cout << "Expected: " << expr << " doesn't throw an exception." << endl;
		cout << "  Actual: it throws." << endl;
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred1Fail(const char *fn, int val1, const char *val1str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		cout << fn << "(" << val1str << ") evaluates to false, where:" << endl;
		cout << val1str << " evaluates to " << val1 << endl;
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred2Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		cout << fn << "(" << val1str << ", " << val2str << ") evaluates to false, where:" << endl;
		cout << val1str << " evaluates to " << val1 << endl;
		cout << val2str << " evaluates to " << val2 << endl;
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred3Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		cout << fn << "(" << val1str << ", " << val2str << ", " << val3str << ") evaluates to false, where:" << endl;
		cout << val1str << " evaluates to " << val1 << endl;
		cout << val2str << " evaluates to " << val2 << endl;
		cout << val3str << " evaluates to " << val3 << endl;
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred4Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int val4, const char *val4str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		cout << fn << "(" << val1str << ", " << val2str << ", " << val3str << ", " << val4str << ") evaluates to false, where:" << endl;
		cout << val1str << " evaluates to " << val1 << endl;
		cout << val2str << " evaluates to " << val2 << endl;
		cout << val3str << " evaluates to " << val3 << endl;
		cout << val4str << " evaluates to " << val4 << endl;
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred5Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int val4, const char *val4str, int val5, const char *val5str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		cout << file << ":" << line << ": Failure" << endl;
		cout << fn << "(" << val1str << ", " << val2str << ", " << val3str << ", " << val4str << ", " << val5str << ") evaluates to false, where:" << endl;
		cout << val1str << " evaluates to " << val1 << endl;
		cout << val2str << " evaluates to " << val2 << endl;
		cout << val3str << " evaluates to " << val3 << endl;
		cout << val4str << " evaluates to " << val4 << endl;
		cout << val5str << " evaluates to " << val5 << endl;

		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
}
